import inspect
import pickle
from pathlib import Path
from re import search
from typing import Any

import msgspec
from addict import Dict as adict
from aiopath import AsyncPath
from itsdangerous import Serializer as SecureSerializer


class AcbEncoder:
    async def process(self):
        if self.action == "load":
            data = self.obj
            if self.path:
                data = await self.path.read_bytes()
            return adict(self.serializer.decode(data, **self.kwargs))
        else:
            data = self.serializer.encode(self.obj, **self.kwargs)
            if self.path:
                return await self.path.write_text(data)
            return data

    def __init__(self):
        self.action = inspect.stack()[1][4][0].split(" = ")[0]
        self.serializers = adict(
            json=msgspec.json, yaml=msgspec.yaml, msgpack=msgspec.msgpack, pickle=pickle
        )
        pickle.encode = pickle.dumps
        pickle.decode = pickle.loads
        for s in self.serializers.keys():
            setattr(self, s, self.__call__)
            setattr(self, f"secure_{s}", self.__call__)

    async def __call__(
        self,
        obj: Any = None,
        path: Path | AsyncPath | str = None,
        sort_keys: bool = False,
        use_list: bool = False,
        secret_key: str = None,
        secure_salt: str = None,
        **kwargs,
    ):
        self.obj = obj
        self.path = None if not path else AsyncPath(path)
        self.secret_key = secret_key
        self.secure_salt = secure_salt
        serializer = search("await\s\w+\.(\w+)\(", inspect.stack()[1][4][0]).group(1)
        self.secure = self.secret_key and self.secure_salt
        serializer = self.serializers[serializer.removeprefix("secure_")]
        self.kwargs = adict(kwargs)
        if self.action == "load" and serializer is msgspec.msgpack:
            self.kwargs.use_list = use_list
        elif self.action == "dump" and serializer is msgspec.yaml:
            self.kwargs.sort_keys = sort_keys
        self.serializer = (
            SecureSerializer(self.secret_key, self.secure_salt, serializer=serializer)
            if self.secure
            else serializer
        )
        return await self.process()


dump = load = AcbEncoder()



from hashlib import blake2b
from pathlib import Path

import arrow
from aiopath import AsyncPath
from google_crc32c import value
from pydantic import BaseModel


class Hash(BaseModel):
    @staticmethod
    async def blake2b(obj: AsyncPath | Path | list | bytes | str) -> str:
        hash_obj = blake2b(digest_size=20)
        if not obj:
            timestamp = arrow.utcnow().float_timestamp
            obj = str(timestamp)
        elif isinstance(obj, AsyncPath | Path):
            obj = AsyncPath(obj) if isinstance(obj, Path) else obj
            obj = await obj.read_bytes()
        elif isinstance(obj, list):
            obj = "".join([str(a) for a in obj])
        if not isinstance(obj, bytes):
            obj = obj.encode()
        hash_obj.update(obj)
        return hash_obj.hexdigest()

    @staticmethod
    def crc32c(obj: Path | bytes | str) -> bytes:
        if isinstance(obj, Path):
            obj = obj.read_text()
        return value(obj.encode())

    @staticmethod
    async def acrc32c(obj: AsyncPath | Path | bytes | str) -> bytes:
        if isinstance(obj, AsyncPath | Path):
            obj = AsyncPath(obj) if isinstance(obj, Path) else obj
            obj = await obj.read_text()
        return value(obj.encode())


hash = Hash()



import typing as t
from pathlib import Path

from encode import load
from encode import dump
from hash import hash
from pydantic import BaseSettings


class AppSettings(BaseSettings):
    formatted: bool = False
    basedir = Path("..").parent
    deployed = True if basedir.name == "app" else False

    class Config:
        extra = "allow"
        arbitrary_types_allowed = True
        json_loads = load.json
        json_dumps = dump.json

    def __init__(self, **values: t.Any):
        super().__init__(**values)
        name = self.__class__.__name__.replace("Settings", "").lower()
        settings = self.basedir / "settings"
        for path in [
            p for p in settings.iterdir() if p.suffix == ".yml" and p.stem == name
        ]:
            yml_settings = path.read_text()
            values = load.yml(yml_settings)
            current_hash = hash.crc32c(yml_settings)
            new_settings = dump.yml(values)
            new_hash = hash.crc32c(new_settings)
            if not self.deployed and not self.formatted and current_hash != new_hash:
                print(f"Changes detected in {path} - formatting...")
                path.write_text(new_settings)
                self.formatted = True
            super().__init__(**values)


class AppConfig(BaseSettings):
    ...

    class Config:
        extra = "allow"


ac = AppConfig()
